<?php
namespace Teknisa\PLA\Modules\Requisicao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Exception;

class Pla00700RequisicaoPorServicoAnalitica extends \Zeedhi\Framework\Controller\Report {

  protected $qr2Strategy;
  protected $novoCodigo;
  protected $connection;
  protected $environment;
  protected $crudService;
  protected $dataSourceManager;
  protected $libUtilities;
  protected $birtStrategy;

  public function __construct($reportService, $novoCodigo, $connection, $environment, $qr2Strategy, $crudService, $dataSourceManager,$libUtilities, $birtStrategy) {
    $this->reportService = $reportService;
    $this->novoCodigo    = $novoCodigo;
    $this->connection    = $connection;
    $this->environment   = $environment;
    $this->qr2Strategy   = $qr2Strategy;
    $this->crudService   = $crudService;
    $this->dataSourceManager   = $dataSourceManager;
    $this->libUtilities  = $libUtilities;
    $this->birtStrategy  = $birtStrategy;
  }

  protected $reportMapping = array (
    'PLA00700' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'NRORG'           => 'P_NRORG'
      )
    ),
    'PLA00701' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'NRORG'           => 'P_NRORG'
      )
    ),
    'PLA00702' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'NRORG'           => 'P_NRORG'
      )
    ),
    'PLA00703' =>
    array (
      'strategy' => 'QR2',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA'
      )
    ),
    'PLA00704' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'NRORG'           => 'P_NRORG'
      )
    ),
    'PLA00705' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'NRORG'           => 'P_NRORG'
      )
    ),
    'PLA00706' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'NRORG'           => 'P_NRORG'
      )
    ),
    'PLA00707' =>
    array (
      'strategy' => 'QR2',
      'parameterMapping' =>
      array (
        'IDQUEBRAPAGINA'  => 'P_IDQUEBRAPAGINA',
        'CDOPERADOR'      => 'P_CDOPERADOR',
        'NMOPERADOR'      => 'P_NMOPERADOR',
        'CDFILIAL'        => 'P_CDFILIAL',
        'NRCONSPLAN'      => 'P_NRCONSPLAN',
        'DTINICIAL'       => 'P_DTINICIAL',
        'DTFINAL'         => 'P_DTFINAL',
        'CDSERVFILI'      => 'P_CDSERVFILI',
        'IDCARD'          => 'P_IDCARD',
        'IDFIXO'          => 'P_IDFIXO',
        'IDPERC'          => 'P_IDPERC',
        'TAMNIV'          => 'P_TAMNIV',
        'NRLINHABRANCA'   => 'P_NRLINHABRANCA',
        'IMPMODOPREPARO'  => 'P_IMPMODOPREPARO'
      )
    )
  );

  public function calculaRequisicaoServicoAnaliticaPla($row) {
    $nrconsplan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$this->environment->getCdFilial(), $this->environment->getNrOrg());
    $row['NRCONSPLAN'] = $nrconsplan;
    if ($row['CDSERVFILI'] == 'T')
      $row['CDSERVFILI'] = $this->retornaServicosConcatenados($row);
    $this->REQUISICAO_ANALITICA_SETOR_PLA($row['__procedure'], $nrconsplan);
  }
  
  public function getServico(DTO\Request $request, DTO\Response $response) {
    $sql = "SELECT
                 SO.CDSERVFILI,
                 SE.NMSERVICO
              FROM
                 SERVOPER SO,
                 SERVICO SE
              WHERE
                 (SO.CDFILIAL   = :CDFILIAL)   AND
                 (SO.CDOPERADOR = :CDOPERADOR) AND
                 ((INSTR(:CDGRUPSERV, SE.CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T')) AND
                 (SE.CDFILIAL   = SO.CDFILIAL)   AND
                 (SE.CDSERVFILI = SO.CDSERVFILI) AND
                 (SE.IDTIPOSERV <> 'A')          AND
                 (SE.IDSERVATIV <> 'N')          AND
                 ((SE.IDTPSERVHOSP IS NULL) OR (SE.IDTPSERVHOSP = '0'))
              ORDER BY
                 SO.CDSERVFILI";
    
    $filter = $request->getParameters()['filter'];
    $params = array(
      "CDOPERADOR" => $filter[0]['value'],
      "CDFILIAL" => $filter[1]['value'],
      "CDGRUPSERV" => $filter[2]['value']
    );
    $result = $this->connection->fetchAll($sql, $params);
 	   
    if (count($result) == 0) {
      $sql = "SELECT
                 'N' as IDMARCADO, 
                 CDSERVFILI,
                 NMSERVICO
              FROM
                 SERVICO
              WHERE
                 (CDFILIAL   = :CDFILIAL) AND
                 ((INSTR(:CDGRUPSERV, CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T')) AND
                 (IDTIPOSERV <> 'A')        AND
                 ((IDSERVATIV IS NULL) OR (IDSERVATIV <> 'N'))    AND
                 ((IDTPSERVHOSP IS NULL) OR (IDTPSERVHOSP = '0'))
              ORDER BY
                 CDSERVFILI";
                 
      $filter = $request->getParameters()['filter'];
      $params = array(
        "CDOPERADOR" => $filter[0]['value'],
        "CDFILIAL" => $filter[1]['value'],
        "CDGRUPSERV" => $filter[2]['value']
      );
      $result = $this->connection->fetchAll($sql, $params);
    }
      
    if (count($filter) > 3) {
      $result = array_filter($result, function($arr) use ($filter) {
        $filter_param = explode("|", $filter[3]['name']);
        $filter_value = str_replace("%", "", $filter[3]['value']);
        return (stripos($arr[$filter_param[0]], $filter_value) !== false) || (stripos($arr[$filter_param[1]], $filter_value) !== false);
      });
    }
    
    $result = array_values($result);
    
    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('pla00700_servico', $result));

  }
 	
 	public function REQUISICAO_ANALITICA_SETOR_PLA($row, $nrconsplan) {
    if($row['SERVICOS'] === 'T') {
      $servicos = $this->modificarServicos($row);
    } else {
      $servicos = $row['SERVICOS'];
    }
    
    //Serviços a ignorar segundo validação do cardápio
    if(isset($row["SERVICOS_A_IGNORAR"])){
      foreach($row['SERVICOS_A_IGNORAR'] as $ignorado){
          $search = $ignorado['CDSERVFILI']."_";
          $servicos = str_replace($search, "", $servicos);
      }
    }
    
    $row['DTHRCALCPR'] = (new \DateTime())->format('d/m/Y H:i:s');

    $procedure = new StoredProcedure($this->connection, 'REQUISICAO_ANALITICA_SETOR_PLA');
    $procedure->addParam(new Param('P_FILIAL',        Param::PARAM_INPUT, $row['CDFILIAL'],     Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_SERVICOS',      Param::PARAM_INPUT, $servicos,            Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_DTINICIAL',     Param::PARAM_INPUT, $row['DTINICIAL'],    Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_DTFINAL',       Param::PARAM_INPUT, $row['DTFINAL'],      Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_IDARQUIVO',     Param::PARAM_INPUT, $row['IDARQUIVO'],    Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_IDPRODUTOS',    Param::PARAM_INPUT, $row['IDPRODUTOS'],   Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_IDSEPPRATO',    Param::PARAM_INPUT, $row['IDSEPPRATO'],   Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_PEACRESCIMO',   Param::PARAM_INPUT, $row['PEACRESCIMO'],  Param::PARAM_TYPE_INT));
    $procedure->addParam(new Param('P_IDARREDOND',    Param::PARAM_INPUT, $row['IDARREDOND'],   Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_NRCONSPLAN',    Param::PARAM_INPUT, $nrconsplan,          Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDPRODUTOINI',  Param::PARAM_INPUT, $row['CDPRODUTOINI'], Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDPRODUTOFIN',  Param::PARAM_INPUT, $row['CDPRODUTOFIN'], Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $this->environment->getCdOperador(),   Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_DTHRCALCPR',    Param::PARAM_INPUT, $row['DTHRCALCPR'],   Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_STDTINICIAL',   Param::PARAM_INPUT, $row['STDTINICIAL'],  Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_STDTFINAL',     Param::PARAM_INPUT, $row['STDTFINAL'],    Param::PARAM_TYPE_STR));

    $procedure->execute();
  }
  
  public function modificarServicos($row) {
    $req = new DTO\Request("", "", $this->environment->getUserId());
    $filter = array(
      array("name" => "CDOPERADOR",
            "operator" => "=",
            "value" => $this->environment->getCdOperador()
      ),
      array("name" => "CDFILIAL",
            "operator" => "=",
            "value" => $row['CDFILIAL']
      ),
      array("name" => "CDGRUPSERV",
            "operator" => "=",
            "value" => $row['CDGRUPSERV']
      )
    );
    $req->setParameter("filter", $filter);
    $res = new DTO\Response();
    $this->getServico($req, $res);
    $res = $res->getDataSets()[0]->getRows();
    $filiais = "_";
    foreach($res as $row){
      $filiais .= $row['CDSERVFILI'].'_';
    }
    return $filiais;
  }

  public function excluiConsplan($nrconsplan) {

    try {
      $param = array("NRCONSPLAN" => $nrconsplan);
      $query = "DELETE FROM CONSPLAN WHERE NRCONSPLAN = :NRCONSPLAN";
      $this->connection->executeQuery($query, $param);
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 1));
    }
  }

  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $row = $request->getRow();
      $row['NRORG'] = $this->environment->getNrOrg();
      
      $this->calculaRequisicaoServicoAnaliticaPla($row);
      //coleta de dados
      $this->libUtilities->saveCollectedData($row['CDFILIAL'],'REQUISICAO');
      $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE'])); 
      $this->qr2Strategy->setQr2Type($row['QR2TYPE']);

  }

  public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
      try {
          $row = $request->getRow();
          $reportCode = $row['__report_name'];
          $strategy = $this->reportMapping[$reportCode]['strategy'];
          if($strategy === 'QR2'){
            $this->excluiConsplan($row['NRCONSPLAN']);
          }
      } catch (\Exception $e) {
        $response->setError(new Error($e->getMessage(), 1));
      }
  }

  public function retornaServicosConcatenados($row) {
    $dataSourceName = "servico_grupserv_filioper";
    $conditions = $this->retornaCondicoesServico($row);
    $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg(new FilterCriteria($dataSourceName, $conditions));
    $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
    $rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
    $servicosConcatenados = array();
    foreach ($rows as $row) {
      $servicosConcatenados[] = $row['CDSERVFILI'];
    }
    return "_" . implode("_", $servicosConcatenados) . "_";
  }

  private function retornaCondicoesServico($row) {
    $cdfilial = isset($row['CDFILIAL']) && $row['CDFILIAL'] ? $row['CDFILIAL'] : 'FILIAL_LOGADA';
    return array(
      array(
        "columnName" => "CDFILIAL",
        "operator" => "=",
        "value" => $cdfilial
      ),
      array(
        "columnName" => "CDOPERADOR",
        "operator" => "=",
        "value" => 'OPERADOR_LOGADO'
      ),
      array(
        "columnName" => "CDGRUPSERV",
        "operator" => "=",
        "value" => $row["CDGRUPSERV"]
      )
    );
  }

}