<?php
namespace Teknisa\PLA\Modules\Requisicao\Controllers;

use \Teknisa\PLA\Modules\Requisicao\Services\Pla01800RequisicaoGeralService;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;

class Pla01800RequisicaoGeral extends \Zeedhi\Framework\Controller\Report {

  protected $pla01800service;
  protected $environment;
  protected $libUtilities;
  protected $session;

  public function __construct(ReportService $reportService, Pla01800RequisicaoGeralService $pla01800service, Session $session, $environment,$libUtilities){
    $this->pla01800service = $pla01800service;
    $this->session = $session;
    $this->environment   = $environment;
    $this->libUtilities  = $libUtilities;
    parent::__construct($reportService);
  }

    /** @var array  */
  protected $reportMapping = array (
    'PLA01800' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        "NRLISTA"          => "P_NRLISTA",
        "LISTAFILIAL"      => "P_CDFILIAL",
        "NRCONSPLAN"       => "P_NRCONSPLAN",
        "DTINICIAL"        => "P_DTINICIAL",
        "DTFINAL"          => "P_DTFINAL",
        "LISTASERVFILI"    => "P_CDSERVFILI",
        "NRTAMANIVEL"      => "P_TAMNIV",
        "IDQUEBRAPAGINA"   => "P_IDQUEBRAPAGINA",
        "NRLINHABRANCA"    => "P_NRLINHABRANCA",
        "L_CDFILIAL"       => "P_LISTAFILIAL",
        "CDOPERADOR"       => "P_CDOPERADOR",
        "NMOPERADOR"       => "P_NMOPERADOR",
        "CDGRUPFILI"       => "P_CDGRUPFILI",
        "IDCONSUMO"        => "P_IDCONSUMO",
        "CDPRODINI"        => "P_CDPRODINI",
        "CDPRODFIN"        => "P_CDPRODFIN",
        "CDPICTPROD"       => "P_CDPICTPROD",
        "CDPRODUTOINICIAL" => "P_CDPRODINIINTERV",
        "NMPRODUTOINICIAL" => "P_NMPRODINIINTERV",
        "CDPRODUTOFINAL"   => "P_CDPRODFININTERV",
        "NMPRODUTOFINAL"   => "P_NMPRODFININTERV",
        "NRORG"            => "P_NRORG"
      ),
    ),
    'PLA01801' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        "NRLISTA"          => "P_NRLISTA",
        "LISTAFILIAL"      => "P_CDFILIAL",
        "NRCONSPLAN"       => "P_NRCONSPLAN",
        "DTINICIAL"        => "P_DTINICIAL",
        "DTFINAL"          => "P_DTFINAL",
        "LISTASERVFILI"    => "P_CDSERVFILI",
        "NRTAMANIVEL"      => "P_TAMNIV",
        "IDQUEBRAPAGINA"   => "P_IDQUEBRAPAGINA",
        "NRLINHABRANCA"    => "P_NRLINHABRANCA",
        "L_CDFILIAL"       => "P_LISTAFILIAL",
        "CDOPERADOR"       => "P_CDOPERADOR",
        "NMOPERADOR"       => "P_NMOPERADOR",
        "CDGRUPFILI"       => "P_CDGRUPFILI",
        "IDCONSUMO"        => "P_IDCONSUMO",
        "CDPRODINI"        => "P_CDPRODINI",
        "CDPRODFIN"        => "P_CDPRODFIN",
        "CDPICTPROD"       => "P_CDPICTPROD",
        "CDPRODUTOINICIAL" => "P_CDPRODINIINTERV",
        "NMPRODUTOINICIAL" => "P_NMPRODINIINTERV",
        "CDPRODUTOFINAL"   => "P_CDPRODFININTERV",
        "NMPRODUTOFINAL"   => "P_NMPRODFININTERV",
        "NRORG"            => "P_NRORG"
      ),
    ),
    'PLA01802' => //Relatório CDS
    array (
      'strategy' => 'QR2',
      'parameterMapping' =>
      array (
        "NRLISTA"          => "P_NRLISTA",
        "LISTAFILIAL"      => "P_CDFILIAL",
        "NRCONSPLAN"       => "P_NRCONSPLAN",
        "DTINICIAL"        => "P_DTINICIAL",
        "DTFINAL"          => "P_DTFINAL",
        "LISTASERVFILI"    => "P_CDSERVFILI",
        "NRTAMANIVEL"      => "P_TAMNIV",
        "IDQUEBRAPAGINA"   => "P_IDQUEBRAPAGINA",
        "NRLINHABRANCA"    => "P_NRLINHABRANCA",
        "L_CDFILIAL"       => "P_LISTAFILIAL",
        "CDOPERADOR"       => "P_CDOPERADOR",
        "NMOPERADOR"       => "P_NMOPERADOR",
        "CDGRUPFILI"       => "P_CDGRUPFILI",
        "IDCONSUMO"        => "P_IDCONSUMO",
        "CDPRODINI"        => "P_CDPRODINI",
        "CDPRODFIN"        => "P_CDPRODFIN",
        "CDPRODUTOINICIAL" => "P_CDPRODINIINTERV",
        "NMPRODUTOINICIAL" => "P_NMPRODINIINTERV",
        "CDPRODUTOFINAL"   => "P_CDPRODFININTERV",
        "NMPRODUTOFINAL"   => "P_NMPRODFININTERV"
      ),
    ),
    'PLA01803' =>
    array (
      'strategy' => 'QR2',
      'parameterMapping' =>
      array (
        "NRLISTA"          => "P_NRLISTA",
        "LISTAFILIAL"      => "P_CDFILIAL",
        "NRCONSPLAN"       => "P_NRCONSPLAN",
        "DTINICIAL"        => "P_DTINICIAL",
        "DTFINAL"          => "P_DTFINAL",
        "LISTASERVFILI"    => "P_CDSERVFILI",
        "NRTAMANIVEL"      => "P_TAMNIV",
        "IDQUEBRAPAGINA"   => "P_IDQUEBRAPAGINA",
        "NRLINHABRANCA"    => "P_NRLINHABRANCA",
        "L_CDFILIAL"       => "P_LISTAFILIAL",
        "CDOPERADOR"       => "P_CDOPERADOR",
        "NMOPERADOR"       => "P_NMOPERADOR",
        "CDGRUPFILI"       => "P_CDGRUPFILI",
        "IDCONSUMO"        => "P_IDCONSUMO",
        "CDPRODINI"        => "P_CDPRODINI",
        "CDPRODFIN"        => "P_CDPRODFIN",
        "CDPRODUTOINICIAL" => "P_CDPRODINIINTERV",
        "NMPRODUTOINICIAL" => "P_NMPRODINIINTERV",
        "CDPRODUTOFINAL"   => "P_CDPRODFININTERV",
        "NMPRODUTOFINAL"   => "P_NMPRODFININTERV"
      ),
    ),
    'PLA01806' =>
    array (
      'strategy' => 'QR2',
      'parameterMapping' =>
      array (
        "NRLISTA"          => "P_NRLISTA",
        "LISTAFILIAL"      => "P_CDFILIAL",
        "NRCONSPLAN"       => "P_NRCONSPLAN",
        "DTINICIAL"        => "P_DTINICIAL",
        "DTFINAL"          => "P_DTFINAL",
        "LISTASERVFILI"    => "P_CDSERVFILI",
        "NRTAMANIVEL"      => "P_TAMNIV",
        "IDQUEBRAPAGINA"   => "P_IDQUEBRAPAGINA",
        "NRLINHABRANCA"    => "P_NRLINHABRANCA",
        "L_CDFILIAL"       => "P_LISTAFILIAL",
        "CDOPERADOR"       => "P_CDOPERADOR",
        "NMOPERADOR"       => "P_NMOPERADOR",
        "CDGRUPFILI"       => "P_CDGRUPFILI",
        "IDCONSUMO"        => "P_IDCONSUMO",
        "TCDTIPOPRAT"      => "P_TCDTIPOPRAT",
        "LISTATIPOPRAT"    => "P_CDTIPOPRAT",
        "CDPRODINI"        => "P_CDPRODINI",
        "CDPRODFIN"        => "P_CDPRODFIN",
        "CDPRODUTOINICIAL" => "P_CDPRODINIINTERV",
        "NMPRODUTOINICIAL" => "P_NMPRODINIINTERV",
        "CDPRODUTOFINAL"   => "P_CDPRODFININTERV",
        "NMPRODUTOFINAL"   => "P_NMPRODFININTERV"
      ),
    ),
  );

  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
    try{
      ini_set('max_execution_time', 999999);
      $row = $request->getRow();
      
      $c_peacrescimo = $row["C_PEACRESCIMO"];
      $nrorg = $this->environment->getNrOrgTrab();
      $row['NRORG'] = $nrorg;
      $cdgrupfili = $row["CDGRUPFILI"];
      $cdoperador = $row["CDOPERADOR"];
      // Listas: LISTAFILIAL, LISTAGRUPSERV, LISTASERVFILI, LISTATIPOPRAT
      
      $cdfilial = $this->pla01800service->retornaFiliais($row);
      $row['L_CDFILIAL'] = $row['T_CDFILIAL'];
      $row['T_CDFILIAL'] = "";
      $row["LISTAFILIAL"] = $cdfilial;
      
      //cria o array com as filiais 
      $arrayFiliais= explode('_',$cdfilial);

      //coleta de dados
      $this->libUtilities->saveCollectedData($arrayFiliais,'REQUISICAO');
      
      
      $cdgrupserv = $this->pla01800service->retornaGrupoServicos($row);
      $row["LISTAGRUPSERV"] = $cdgrupserv;
      $cdservfili = $this->pla01800service->retornaServicos($row);
      $row["LISTASERVFILI"] = $cdservfili;
      $cdtipoprat = $this->pla01800service->retornaTipoPrato($row);
      $row["LISTATIPOPRAT"] = $cdtipoprat;
      $row["T_CDSERVFILI"] = ($row["T_CDSERVFILI"] == "T" ? "S" : "N");
      $row["NRCONSPLAN"] = $this->session->get("NRCONSPLAN");
      $row['CDPICTPROD'] = $this->pla01800service->getReportProductMask();
      $row["SERVICOFILIAL"] = array_map(function($currentItem){
        return $currentItem["CDFILIAL"].$currentItem["CDSERVFILI"];
      }, $row["SERVICOFILIAL"]);
      //Serviços a ignorar conforme rotina de validação do cardápio
      if(isset($row['SERVICOS_A_IGNORAR'])){
        foreach($row['SERVICOS_A_IGNORAR'] as $ignorado){
          $valor = $ignorado["CDFILIAL"].$ignorado["CDSERVFILI"];
          $index = array_search($valor, $row["SERVICOFILIAL"]);
          if ($index !== false) {
            unset($row["SERVICOFILIAL"][$index]);
          }
        }
      }
      
      if(!isset($row['NRLINHABRANCA']) || !$row['NRLINHABRANCA']){
        $row['NRLINHABRANCA'] = 0;
      }
      if($c_peacrescimo == 'S' || empty($row["NRCONSPLAN"])){
        // $row é passado por referência aqui, sendo assim o método calculaRequisicaoGeral consegue alterá-la
        
        $stResultado = $this->pla01800service->calculaRequisicaoGeral($row);
        $this->session->set("NRCONSPLAN" ,$row["NRCONSPLAN"]);
        if(!empty($stResultado)){
          throw new \Exception($stResultado);
        }
      }else{
        $nrlista = $this->pla01800service->montaListaFiliServ($nrorg, $row["SERVICOFILIAL"]);
        $row["LISTASERVFILI"] = $nrlista;
        if($row['CD_REPORT_NAME'] == 'PLA01806'){
          $result = $this->pla01800repository->verificaLista($nrlista);
          if(!empty($result)){
             throw new \Exception("Existe(m) serviço(s) que não utiliza(m) requisição/retirada por prato.");
          }
        }
        $row["NRLISTA"] = $nrlista;
      }
      if($row["CDPRODFIN"] != "zzzzzzzzzz")
        $row["CDPRODUTOFINAL"] = str_replace("z", "", $row["CDPRODFIN"]);
    } catch(\Exception $e){
        $response->setError(new Error($e->getMessage(), 1));
        throw $e;
    }
  }

  public function deletaConsplan(DTO\Request\Row $request, DTO\Response $response){
    $nrconsplan = $this->session->get("NRCONSPLAN");
    $this->pla01800service->clearConsplan($nrconsplan);
    $this->session->remove("NRCONSPLAN");
  }

}